#!/bin/sh

# Function to get string segment with seperator '='
# e.g. $1 = "APP_HOME=/usr/local/obm APP_HOME=/root/Documents/obm" , $2 = 3 
# then this function returns /root/Documents/obm
get_column_value()
{
    STRING_PATTERN="$1"
    COL_NUM=$2
    echo $STRING_PATTERN | awk -F"=" '{print $'"${COL_NUM}"'}'
}

# ------------------------- Retrieve APP_HOME ----------------------------------
cd `dirname "$0"`

SCRIPT_NAME="$0"
MODE="manual"
if [ ! -z "$1" ]; then
    if [ "$1" = "rpm" ]; then
        MODE="rpm"
    fi
fi
APP_HOME=`pwd`
if [ ! -z "$2" ]; then
    APP_HOME="$2"
fi
PRIVILEGE_CHK=""
if [ ! -z "$3" ]; then
    PRIVILEGE_CHK="$3"
fi
USER_JVM_HOME=""
APP_BIN=$APP_HOME/bin
TAR_FOLDER_NAME=_components
TAR_FOLDER="$APP_HOME/$TAR_FOLDER_NAME"

# get arch of this system
PLATFORM_NAME=""
case "`uname`" in
    Solaris* | SunOS*)
        PLATFORM_NAME="solaris"
        case "`isainfo -b`" in
            32 | 64)
                OS_ARCH="32"
                ;;
            # amd64 is NOT support in Solaris / SunOS
            #64)
            #    OS_ARCH="64"
            #    ;;
            *)
                OS_ARCH="Others"
                ;;
        esac
        ;;
    *)
        PLATFORM_NAME="others"
        case "`uname -m`" in
            i[3-6]86)
                OS_ARCH="32"
                ;;
            x86_64 | amd64)
                OS_ARCH="64"
                ;;
            *)
                OS_ARCH="Others"
                ;;
        esac
        ;;
esac

# Get INIT_PATH to get APP_HOME from running service
case "`uname`" in 
    Linux*)      INSTALL_TYPE=NIX ;;
    Solaris*)    INSTALL_TYPE=NIX ;;
    SunOS*)      INSTALL_TYPE=NIX ;;
    OpenBSD*)    INSTALL_TYPE=OPENBSD ;;
    *BSD*)       INSTALL_TYPE=BSD ;;
    **)          INSTALL_TYPE=NOT_SUPPORTED ;;
esac

# Initialize INIT_PATH to empty
INIT_PATH=""
# Set INIT_PATH for different OS
if [ $INSTALL_TYPE = "NIX" ]; then
    if [ -z "$INIT_PATH" ]; then
        if [ -d /etc/init.d ]; then
            INIT_PATH=/etc/init.d
        elif [ -d /etc/rc.d/init.d ]; then
            INIT_PATH=/etc/rc.d/init.d
        fi
    fi
    
elif [ $INSTALL_TYPE = "BSD" ]; then
    INIT_PATH=/usr/local/etc/rc.d
    
elif [ $INSTALL_TYPE = "OPENBSD" ]; then
    # get OpenBSD version
    OPENBSD_VERSION=`uname -r | cut -f1 -d"."`

    if [ $OPENBSD_VERSION -ge 5 ]
    then
        INIT_PATH=/etc/rc.d
    else
        INIT_PATH=/etc
    fi
fi

# -------------------- Print Logging Message Header ----------------------------
echo "Log Time: `date`"

if [ ! -d "$TAR_FOLDER" ] ; then
    echo "$TAR_FOLDER does not exist, installer will terminate"
    exit 1
fi

# ------------ Backup user setting files ------------------
# Get old APP_HOME
if [ -f "$APP_HOME/bin/uninstall.sh" ]; then
    APP_HOME_OLD=$APP_HOME
elif [ ! -z "$INIT_PATH" ]; then
    OBM_SCRIPT_FILE="$INIT_PATH/cbpscheduler"
    if [ -f $OBM_SCRIPT_FILE ] ; then
        OBM_HOME_INFO=`cat $OBM_SCRIPT_FILE | grep APP_HOME=`
        APP_HOME_OLD=`get_column_value "$OBM_HOME_INFO" 3`
    fi
fi

if [ -z "$APP_HOME_OLD" ]; then
    echo "  No old application found, begin fresh install"
else
    if [ -f "$APP_HOME_OLD/bin/uninstall.sh" ]; then
        echo "  Backup user setting files"

        # directory for temporary keeping backup files
        TEMP_DIR="_tmp"
        BACKUP_DIR="$APP_HOME/$TEMP_DIR"
        if [ -d "$BACKUP_DIR" ]; then
            echo "  Remove previous cache"
            rm -rf "$BACKUP_DIR"
        fi
        mkdir "$BACKUP_DIR"

        # directory to be backup / restore
        DIR_NAME="log"
        for DIR in $DIR_NAME; do
            if [ -d "$APP_HOME_OLD/$DIR" ]; then
                echo "    Backup $DIR"
                mv -f "$APP_HOME_OLD/$DIR" "$BACKUP_DIR/$DIR"
            fi
        done

        # file to be backup / restore
        FILE_NAME="home.txt config.ini afc.opt cb.opt"
        for FILE in $FILE_NAME; do
            if [ -f "$APP_HOME_OLD/$FILE" ]; then
                echo "    Backup $FILE"
                mv -f "$APP_HOME_OLD/$FILE" "$BACKUP_DIR/$FILE"
            fi
        done
        echo "  Backup finished"

        # Run uninstall to stop Scheduler service
        echo "  Uninstall previous version..."
        "$APP_HOME_OLD/bin/uninstall.sh" "no_privilege_check"

        echo "  Remove previous application files"
        APP_DATA="`ls "$APP_HOME_OLD" | sort `"
        for DATA in $APP_DATA; do
            if [ "$DATA" != "$TEMP_DIR" ] && [ "$DATA" != "$TAR_FOLDER_NAME" ] ; then
                if [ -d "$APP_HOME_OLD/$DATA" ] || [ -L "$APP_HOME_OLD/$DATA" ]; then
                    echo "    Remove directory $DATA"
                    rm -rf "$APP_HOME_OLD/$DATA"
                else
                    if [ -z "`echo \"$APP_HOME_OLD/$DATA\" | grep ${SCRIPT_NAME}$`" ]; then
                        if [ -f "$APP_HOME_OLD/$DATA" ]; then
                            echo "    Remove file $DATA"
                            rm "$APP_HOME_OLD/$DATA"
                        fi
                    fi
                fi
            fi
        done
        echo "  Remove application files finished"
    else
        echo "  No old application found, begin fresh install"
    fi
fi

# ----------------------- Untar jvm and component file --------------------------------
cd "$TAR_FOLDER"
for entry in "$TAR_FOLDER"/*
do
    case "$entry" in
        *"/jre-"* )
            if [ ! -d "$APP_HOME/jvm" ] ; then
                echo "$APP_HOME/jvm does not exist, creating this directory"
                mkdir "$APP_HOME/jvm" > /dev/null
            fi
            echo "Untar JVM file : $entry"
            if [ "$PLATFORM_NAME" = "solaris" ] ; then
                cd "$APP_HOME/jvm"
                gzip -cdf "$entry" | /usr/sfw/bin/gtar xf - > /dev/null
                cd "$TAR_FOLDER"
            elif [ "$PLATFORM_NAME" = "others" ] ; then
                tar -xzf "$entry" -C "$APP_HOME/jvm" > /dev/null
            fi
            ;;
        *)
            echo "Untar component file : $entry"
            if [ "$PLATFORM_NAME" = "solaris" ] ; then
                cd "$APP_HOME"
                gzip -cdf "$entry" | /usr/sfw/bin/gtar xf - > /dev/null
                cd "$TAR_FOLDER"
            elif [ "$PLATFORM_NAME" = "others" ] ; then
                tar -xzf "$entry" -C "$APP_HOME" > /dev/null
            fi
            ;;
    esac
done

cd "$APP_HOME"
echo "Untar jvm and component files finished"
rm -rf "$TAR_FOLDER"

# ----------------------- Verify if JVM home exist --------------------------------
JVM_HOME=$APP_HOME/jvm
if [ ! -d "$JVM_HOME" ]; then
    if [ -z "$USER_JVM_HOME" ]; then
        echo "Please enter your java 1.7 home: "
        read USER_JVM_HOME
    fi
    if [ -z "$USER_JVM_HOME" ]; then
        echo "Invalid java home!"
        exit 1
    fi
    if [ ! -d "$USER_JVM_HOME" ]; then
        echo "$USER_JVM_HOME does not exist!"
        exit 1
    fi
    JVM_SECURITY="$APP_HOME/util/jvm/lib/security"
    if [ -d "$JVM_SECURITY" ]; then
        echo "Copy java 1.7 from $USER_JVM_HOME"
        cp -rf "$USER_JVM_HOME" "$JVM_HOME"
        cp -rf $JVM_SECURITY/* $JVM_HOME/lib/security
    else
        echo "Create symbolic link from $USER_JVM_HOME"
        ln -sf "$USER_JVM_HOME" "$JVM_HOME"
    fi
fi

# ----------------------- Install Application --------------------------------
TIMESTAMP_LOG=`date '+%y%m%d%H%M%S'`
INSTALL_LOG="$APP_HOME/install-$TIMESTAMP_LOG.log"
INSTALL_SCRIPT="$APP_BIN/install.sh"
if [ ! -f "$INSTALL_SCRIPT" ]; then
    echo "Install script not exist"
    exit 1
fi
echo "Install Application Path: $APP_HOME"
sh "$INSTALL_SCRIPT" "$PRIVILEGE_CHK" >> "$INSTALL_LOG"

# ----------------------- Restore user data files --------------------------------
# directory for temporary keeping backup files
TEMP_DIR="_tmp"
BACKUP_DIR="$APP_HOME/$TEMP_DIR"

if [ -d "$BACKUP_DIR" ]; then
    echo "  Restore Previous Setting backup ..."

    # directory to be backup / restore
    DIR_NAME="log"
    for DIR in $DIR_NAME; do
        if [ -d "$BACKUP_DIR/$DIR" ]; then
            echo "    Restore $DIR"
            mv -f "$BACKUP_DIR/$DIR" "$APP_HOME/$DIR"
        fi
    done

    # migration logic for properties file 'afc.opt' and 'cb.opt'.
    FILEPATH_UTIL="$APP_HOME/util/lib/installer-util.jar"
    CLASSNAME_UTIL=PropertiesMigration

    # file to be backup / restore
    FILE_NAME="home.txt config.ini afc.opt cb.opt"
    for FILE in $FILE_NAME; do
        if [ -f "$BACKUP_DIR/$FILE" ]; then
            # afc.opt AND cb.opt need perform migration
            if [ "$FILE" = "afc.opt" ]; then
                echo "    Restore application opt file (afc.opt)"
                FILEPATH_OUTPUT="$BACKUP_DIR/afc.opt.new"
                FILEPATH_DEFAULT="$BACKUP_DIR/afc.opt"
                FILEPATH_CUSTOM="$APP_HOME/afc.opt"
                java -cp "${FILEPATH_UTIL}" "${CLASSNAME_UTIL}" "${FILEPATH_OUTPUT}" "${FILEPATH_DEFAULT}" "${FILEPATH_CUSTOM}"
                mv -f "${FILEPATH_OUTPUT}" "${FILEPATH_CUSTOM}"
            else
                if [ "$FILE" = "cb.opt" ]; then
                    echo "    Restore application opt file (cb.opt)"
                    FILEPATH_OUTPUT="$BACKUP_DIR/cb.opt.new"
                    FILEPATH_DEFAULT="$BACKUP_DIR/cb.opt"
                    FILEPATH_CUSTOM="$APP_HOME/cb.opt"
                    java -cp "${FILEPATH_UTIL}" "${CLASSNAME_UTIL}" "${FILEPATH_OUTPUT}" "${FILEPATH_DEFAULT}" "${FILEPATH_CUSTOM}"
                    mv -f "${FILEPATH_OUTPUT}" "${FILEPATH_CUSTOM}"
                else
                    echo "    Restore $FILE"
                    mv -f "$BACKUP_DIR/$FILE" "$APP_HOME/$FILE"
                fi
            fi
        fi
    done

    rm -rf "$BACKUP_DIR"
    echo "  Previous Setting backup restored"
fi

echo "Done"
exit 0
